@@ -1,4 +1,5 @@
 import itertools
+from collections import deque
 from collections.abc import Callable, Generator, Iterator
 from dataclasses import dataclass, field
 from inspect import isclass, isgeneratorfunction
@@ -13,8 +14,8 @@ type HookFunction[**P, T] = Callable[P, T] | HookGeneratorFunction[P, T]
 
 @dataclass(eq=False, frozen=True, slots=True)
 class Hook[**P, T]:
-    __functions: list[HookFunction[P, T]] = field(
-        default_factory=list,
+    __functions: deque[HookFunction[P, T]] = field(
+        default_factory=deque,
         init=False,
         repr=False,
     )
@@ -35,7 +36,7 @@ class Hook[**P, T]:
         return iter(self.__functions)
 
     def add(self, *functions: HookFunction[P, T]) -> Self:
-        self.__functions.extend(reversed(functions))
+        self.__functions.extendleft(functions)
         return self
 
     @classmethod
